From 0e88d5ae351dc6994c76fe3f56077367c47fe163 Mon Sep 17 00:00:00 2001
From: Jean-Paul Saman <jpsaman@videolan.org>
Date: Thu, 10 May 2012 21:10:52 +0200
Subject: [PATCH 2/2] video_filter/scene.c: Limit the maximum time worth of
 pictures in the fifo.

Limit the size of the internal picture fifo in time. The option
--scene-buffer-time provides the maximum amount of time worth of
pictures that can be in the fifo. The delta is taken from the first
and the last picture in the fifo.
---
 modules/video_filter/scene.c |   24 ++++++++++++++++++++++--
 1 files changed, 22 insertions(+), 2 deletions(-)

diff --git a/modules/video_filter/scene.c b/modules/video_filter/scene.c
index 449300a..e7e6ada 100644
--- a/modules/video_filter/scene.c
+++ b/modules/video_filter/scene.c
@@ -89,6 +89,11 @@ static void *WriteThread( void * );
                             "creating one file per image. In this case, " \
                              "the number is not appended to the filename." )
 
+#define BUFFER_TIME_TEXT N_( "Maximum picture duration in buffer (default: 5000 ms)" )
+#define BUFFER_TIME_LONGTEXT N_( "Buffer a maximum of time (in ms) worth of pictures " \
+                                 "in the internal buffer. Setting this to 0 ms allows " \
+                                 "for unlimited growth of buffer.")
+
 #define SCENE_HELP N_("Send your video to picture files")
 #define CFG_PREFIX "scene-"
 
@@ -114,6 +119,9 @@ vlc_module_begin ()
     add_bool(    CFG_PREFIX "replace", false,
                  REPLACE_TEXT, REPLACE_LONGTEXT, false )
 
+    add_integer( CFG_PREFIX "buffer-time", 5000,
+                 BUFFER_TIME_TEXT, BUFFER_TIME_LONGTEXT, false)
+
     /* Snapshot method */
     add_integer( CFG_PREFIX "ratio", 50,
                  RATIO_TEXT, RATIO_LONGTEXT, false )
@@ -122,12 +130,12 @@ vlc_module_begin ()
 vlc_module_end ()
 
 static const char *const ppsz_vfilter_options[] = {
-    "format", "width", "height", "ratio", "prefix", "path", "replace", NULL
+    "format", "width", "height", "ratio", "prefix", "path", "replace", "buffer-time", NULL
 };
 
 typedef struct scene_t {
     picture_fifo_t  *fifo;
-
+    mtime_t         delta;
     vlc_mutex_t     lock;
     vlc_cond_t      wait;
     bool            empty;
@@ -207,6 +215,8 @@ static int Create( vlc_object_t *p_this )
 
     p_sys->scene.stop = false;
     p_sys->scene.empty = true;
+    p_sys->scene.delta = var_CreateGetInteger( p_this, CFG_PREFIX "buffer-time" )
+                         * INT64_C(1000);
 
     vlc_mutex_init( &p_sys->scene.lock );
     vlc_cond_init( &p_sys->scene.wait );
@@ -285,6 +295,16 @@ static void SnapshotRatio( filter_t *p_filter, picture_t *p_pic )
     }
     p_sys->i_frames++;
 
+    mtime_t delta = picture_fifo_GetDeltaDate( p_sys->scene.fifo );
+    if( p_sys->scene.delta > 0 &&
+        delta >= p_sys->scene.delta )
+    {
+        msg_Warn( p_filter, "Write buffer full (%"PRId64" ms), flushing buffer",
+                  delta/INT64_C(1000) );
+        picture_fifo_Flush( p_sys->scene.fifo, p_pic->date, true );
+        return;
+    }
+
     picture_t *p_newpic = picture_NewFromFormat( &p_pic->format );
     if( p_newpic )
     {
-- 
1.7.7.6

